# coding=utf-8
import hashlib
import random


class StringUtils(object):

    @staticmethod
    def randomStr(length):
        seed = "1234567890abcdefghijklmnopqrstuvwxyz"
        sa = []
        for i in range(length):
            sa.append(random.choice(seed))
        return ''.join(sa)

    @staticmethod
    def randomNum(length):
        seed = "1234567890"
        sa = []
        for i in range(length):
            sa.append(random.choice(seed))
        return ''.join(sa)

    @staticmethod
    def md5(data):
        h = hashlib.md5()
        h.update(data)
        return h.hexdigest()

    @staticmethod
    def phoneToNick(phone):
        if len(phone) > 4:
            return "zzyl_" + phone[-4:]
        else:
            return phone

    @staticmethod
    # 将字典转成字符串
    def dict2str(d):
        s = ''
        for i in d:
            s = s + i + ': ' + d[i] + '\r\n'
        return s



# coding=utf-8
import traceback


def init():
    global _global_dict
    _global_dict = {}


def set_v(name, value):
    _global_dict[name] = value


def get_v(name, def_value=None):
    try:
        return _global_dict[name]
    except KeyError:
        return def_value


def del_v(name):
    try:
        del _global_dict[name]
    except:
        print traceback.print_exc()



import re


class TcpCount(object):

    def get_contents(self, file):
        f = open(file, "r").readlines()
        return f

    def covert_number(self, number):
        value = hex(number)
        return value

    def get_connect(self, port):
        ESTABLISHED = []
        port = str(self.covert_number(port)).replace("0x", "").upper()
        for i in self.get_contents('/proc/net/tcp'):
            c = i.split()
            p = re.findall(r":.*%(port)s" % vars(), c[1])

            if c[3] == "01" and p:
                ESTABLISHED.append(c[3])
        return len(ESTABLISHED)

    def get_info(self, port1):
        ESTABLISHED = []
        SYN_SENT = []
        SYN_RECV = []
        FIN_WAIT1 = []
        FIN_WAIT2 = []
        TIME_WAIT = []
        CLOSE = []
        CLOSE_WAIT = []
        CLOSING = []
        port = str(self.covert_number(port1)).replace("0x", "").upper()
        for i in self.get_contents('/proc/net/tcp'):
            c = i.split()
            p = re.findall(r":.*%(port)s" % vars(), c[1])

            if c[3] == "01" and p:
                ESTABLISHED.append(c[3])
            elif c[3] == "02" and p:
                SYN_SENT.append(c[3])
            elif c[3] == "03" and p:
                SYN_RECV.append(c[3])
            elif c[3] == '04' and p:
                FIN_WAIT1.append(c[3])
            elif c[3] == '05' and p:
                FIN_WAIT2.append(c[3])
            elif c[3] == '06' and p:
                TIME_WAIT.append(c[3])
            elif c[3] == '07' and p:
                CLOSE.append(c[3])
            elif c[3] == '08' and p:
                CLOSE_WAIT.append(c[3])
            elif c[3] == '0B' and p:
                CLOSING.append(c[3])

        return str(port1) + "ESTABLISHED:\t" + str(len(ESTABLISHED)) + " \nSYN_SENT:\t" + str(len(SYN_SENT)) + \
               "\nSYN_RECV:\t" + str(len(SYN_RECV)) + " \nFIN_WAIT1:\t" + str(len(FIN_WAIT1)) + \
               "\nFIN_WAIT2:\t" + str(len(FIN_WAIT2)) + " \nTIME_WAIT:\t" + str(len(TIME_WAIT)) + \
               "\nCLOSE:  \t" + str(len(CLOSE)) + " \nCLOSE_WAIT:\t" + str(len(CLOSE_WAIT)) + \
               "\nCLOSING:\t" + str(len(CLOSING))



from Queue import Queue, Full, Empty
from time import time


class BatchQueue(Queue):
    def putall(self, list, block=True, timeout=None):
        """Put an item into the queue.

        If optional args 'block' is true and 'timeout' is None (the default),
        block if necessary until a free slot is available. If 'timeout' is
        a non-negative number, it blocks at most 'timeout' seconds and raises
        the Full exception if no free slot was available within that time.
        Otherwise ('block' is false), put an item on the queue if a free slot
        is immediately available, else raise the Full exception ('timeout'
        is ignored in that case).
        """
        self.not_full.acquire()
        try:
            if self.maxsize > 0:
                if not block:
                    if self._qsize() == self.maxsize:
                        raise Full
                elif timeout is None:
                    while self._qsize() == self.maxsize:
                        self.not_full.wait()
                elif timeout < 0:
                    raise ValueError("'timeout' must be a non-negative number")
                else:
                    endtime = time() + timeout
                    while self._qsize() == self.maxsize:
                        remaining = endtime - time()
                        if remaining <= 0.0:
                            raise Full
                        self.not_full.wait(remaining)
            self.queue.extend(list)
            self.unfinished_tasks += len(list)
            self.not_empty.notify()
        finally:
            self.not_full.release()

    def getall(self, num, block=True, timeout=None):
        """Remove and return an item from the queue.

        If optional args 'block' is true and 'timeout' is None (the default),
        block if necessary until an item is available. If 'timeout' is
        a non-negative number, it blocks at most 'timeout' seconds and raises
        the Empty exception if no item was available within that time.
        Otherwise ('block' is false), return an item if one is immediately
        available, else raise the Empty exception ('timeout' is ignored
        in that case).
        """
        list = []
        self.not_empty.acquire()
        try:
            if not block:
                if not self._qsize():
                    raise Empty
            elif timeout is None:
                while not self._qsize():
                    self.not_empty.wait()
            elif timeout < 0:
                raise ValueError("'timeout' must be a non-negative number")
            else:
                endtime = time() + timeout
                while not self._qsize():
                    remaining = endtime - time()
                    if remaining <= 0.0:
                        raise Empty
                    self.not_empty.wait(remaining)
            while self._qsize() and num > len(list):
                list.append(self._get())
            self.not_full.notify()
            return list
        finally:
            self.not_empty.release()



# coding=utf-8
import socket
import threading
import traceback
from Queue import Queue

from pycore.http.handle import HttpRequest
from pycore.utils.logger_utils import LoggerUtils

logger = LoggerUtils('http_server').logger


class WorkThread(threading.Thread):
    def __init__(self, work_queue):
        super(WorkThread, self).__init__()
        self.work_queue = work_queue
        self.daemon = True

    def run(self):
        while True:
            func, args = self.work_queue.get()
            func(*args)
            self.work_queue.task_done()


# 线程池
class ThreadPoolManger:
    def __init__(self, thread_number):
        self.thread_number = thread_number
        self.work_queue = Queue()
        for i in range(self.thread_number):  # 生成一些线程来执行任务
            thread = WorkThread(self.work_queue)
            thread.start()

    def add_work(self, func, *args):
        self.work_queue.put((func, args))


def tcp_link(sock, addr, handle=HttpRequest):
    rfile = sock.makefile('rb', -1)
    http_req = handle(addr)
    try:
        http_req.passRequest(rfile)
    except:
        logger.exception(traceback.format_exc())
    sock.send(http_req.getResponse())
    sock.close()


def start_server(port, pool, handle=HttpRequest):
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.bind(('', port))
    s.listen(10)
    thread_pool = ThreadPoolManger(pool)
    while True:
        try:
            sock, addr = s.accept()
            thread_pool.add_work(tcp_link, *(sock, addr, handle))
        except:
            break



# coding=utf-8
import random
import threading
import time

import cPickle
import redis

import pycore.data.entity.globalvar as gl
from pycore.data.entity import config


class RedisUtils(object):

    def __init__(self):
        self.__pool = redis.ConnectionPool(host=config.get("redis", "host"), port=int(config.get("redis", "port")),
                                           db=0, password=config.get("redis", "pass"))
        self.__redis = redis.Redis(connection_pool=self.__pool)
        redis_ping = threading.Thread(target=self._ping, name="redis_ping")
        redis_ping.start()

    def lock(self, key):
        """
        : 加锁
        :param key:
        :return:
        """
        t = time.time()
        while True:
            if self.__redis.setnx(key, time.strftime("%Y%m%d%H%M%S", time.localtime())) == 1:
                self.__redis.expire(key, 5)
                return True
            time.sleep(random.randint(1, 5) / 1000.0)

    def unlock(self, key):
        """
        : 解锁
        :param key:
        :return:
        """
        gl.get_v("serverlogger").logger.info("解锁%s" % key)
        self.__redis.delete(key)

    def setobj(self, key, obj):
        """
        : 存入对象
        :param key:
        :param obj:
        :return:
        """
        self.__redis.set(key, cPickle.dumps(obj, -1))

    def set(self, key, str):
        """
        : 存入数组
        :param key:
        :param obj:
        :return:
        """
        self.__redis.set(key, str)

    def setexo(self, key, obj, ex):
        """
        : 存入数组
        :param key:
        :param obj:
        :param ex:
        :return:
        """
        self.__redis.setex(key, ex, cPickle.dumps(obj, -1))

    def setex(self, key, obj, ex):
        """
        : 存入数组
        :param key:
        :param obj:
        :param ex:
        :return:
        """
        self.__redis.setex(key, ex, obj)

    def getobj(self, key):
        """
        : 取出对象
        :param key:
        :return:
        """
        jsons = self.__redis.get(key)
        obj = cPickle.loads(jsons)
        return obj

    def get(self, key):
        """
        : 取出数组
        :param key:
        :return:
        """
        return self.__redis.get(key)

    def delobj(self, key):
        """
        : 删除对象
        :param key:
        :return:
        """
        self.__redis.delete(key)

    def exists(self, key):
        return self.__redis.exists(key)

    def keys(self, key):
        return self.__redis.keys(key)

    def publish(self, chan, msg):
        self.__redis.publish(chan, msg)
        return True

    def subscribe(self, chan):
        pub = self.__redis.pubsub()
        pub.subscribe(chan)
        pub.parse_response()
        return pub

    def sigleSubscribe(self, subject, handle):
        redis_sub = self.subscribe(subject)
        while True:
            msg = redis_sub.parse_response()
            handle(msg[2])

    def startSubscribe(self, subject, handle):
        for i in range(0, len(subject)):
            threading.Thread(target=self.sigleSubscribe, args=(subject[i], handle[i],), name=subject[i]).start()

    def _ping(self):
        while True:
            time.sleep(60)
            # 尝试向redis-server发一条消息
            if not self.__redis.ping():
                print("oops~ redis-server get lost. call him back now!")
                del self.__pool
                del self.__redis
                self.__pool = redis.ConnectionPool(host=config.get("redis", "host"),
                                                   port=int(config.get("redis", "port")),
                                                   db=0)
                self.__redis = redis.Redis(connection_pool=self.__pool)



#!/usr/bin/env python
# coding: utf-8
from setuptools import setup, find_packages

setup(
    name='pycore-utils',
    version='0.0.24',
    author='pycore',
    author_email='pycore_team@gmail.com',
    description='python utils',
    packages=find_packages(), install_requires=['pymysql', 'redis']
)



# coding=utf-8
import mimetools

from pycore.http import ErrorCode
from pycore.utils.logger_utils import LoggerUtils
from pycore.utils.stringutils import StringUtils

logger = LoggerUtils('access').logger


class HttpRequest(object):

    def __init__(self, addr):
        self.method = None
        self.url = None
        self.protocol = None
        self.head = dict()
        self.request_data = dict()
        self.response_line = ErrorCode.NOT_FOUND
        self.response_head = dict()
        self.response_body = 'No Something To Do'
        self.session = None
        self.addr = addr

    def log_message(self, formatstr, *args):
        logger.info("%s:%d - - %s\n" % (self.addr[0], self.addr[1], formatstr % args))

    def passRequestLine(self, request_line):
        header_list = request_line.split(' ')
        self.log_message(request_line)
        self.method = header_list[0].upper()
        self.url = header_list[1]
        self.protocol = header_list[2]

    def passRequestHead(self, request_head):
        head_options = request_head.split('\r\n')
        for option in head_options:
            key, val = option.split(': ', 1)
            self.head[key] = val

    def passRequestData(self, body):
        if self.method == 'POST':
            self.request_data = {}
            self.log_message("param:%s", str(body))
            parameters = body.split('&')  # 每一行是一个字段
            for i in parameters:
                if i == '':
                    continue
                key, val = i.split('=', 1)
                self.request_data[key] = val
        if self.method == 'GET':
            if self.url.find('?') != -1:  # 含有参数的get
                self.request_data = {}
                req = self.url.split('?', 1)[1]
                s_url = self.url.split('?', 1)[0]
                self.log_message("param:%s", str(req))
                parameters = req.split('&')
                for i in parameters:
                    key, val = i.split('=', 1)
                    self.request_data[key] = val
                self.url = s_url

    def passRequest(self, rfile):
        request_line = rfile.readline(65537)
        request_line = request_line.decode('utf-8')
        self.passRequestLine(request_line)

        self.head = mimetools.Message(rfile, 0)
        data = ''
        if "content-length" in self.head:
            length = self.head.getheader('content-length')
            length = int(length)
            data = rfile.read(length)
            while len(data) < length:
                data += rfile.read(length - len(data))
        rfile.close()
        self.passRequestData(data)
        self.do()

    def do(self):
        pass

    def getResponse(self):
        response = self.response_line + StringUtils.dict2str(self.response_head) + '\r\n' + self.response_body
        self.log_message("return:%s", str(response))
        return response
